/*
 * Copyright 2012 Google Inc.
 *
 * Use of this source code is governed by a BSD-style license that can be
 * found in the LICENSE file.
 */

#ifndef SkImages_DEFINED
#define SkImages_DEFINED

class SkImages {
public:
    /**
     * Initializes flattenables in the images project.
     */
    static void InitializeFlattenables();
};

#endif
