/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(6c1034f0-1dd2-11b2-aa14-e6657ed7bb0b)]
interface nsIUserInfo : nsISupports
{
    /* these are things the system may know about the current user */

    readonly attribute wstring fullname;

    readonly attribute string emailAddress;

    /* should this be a wstring? */
    readonly attribute string username;

    readonly attribute string domain;
};

%{C++

// 14c13684-1dd2-11b2-9463-bb10ba742554
#define NS_USERINFO_CID \
{  0x14c13684, 0x1dd2, 0x11b2, \
  {0x94, 0x63, 0xbb, 0x10, 0xba, 0x74, 0x25, 0x54}}

#define NS_USERINFO_CONTRACTID "@mozilla.org/userinfo;1"

%}
