/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */

#ifndef mozilla_dom_DeviceMotionEventBinding_h
#define mozilla_dom_DeviceMotionEventBinding_h

#include "EventBinding.h"
#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/Nullable.h"

namespace mozilla {
namespace dom {

class DeviceAcceleration;
struct DeviceAccelerationAtoms;
struct DeviceAccelerationInit;
struct DeviceAccelerationInitAtoms;
class DeviceMotionEvent;
struct DeviceMotionEventAtoms;
struct DeviceMotionEventInitAtoms;
class DeviceRotationRate;
struct DeviceRotationRateAtoms;
struct DeviceRotationRateInit;
struct DeviceRotationRateInitAtoms;
class NativePropertyHooks;
class ProtoAndIfaceCache;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

struct DeviceAccelerationInit : public DictionaryBase
{
  Nullable<double> mX;
  Nullable<double> mY;
  Nullable<double> mZ;

  DeviceAccelerationInit();

  explicit inline DeviceAccelerationInit(const DeviceAccelerationInit& aOther)
  {
    *this = aOther;
  }

protected:
  explicit inline DeviceAccelerationInit(int )
  {
    // Do nothing here; this is used by our "Fast" subclass
  }

private:
  static bool
  InitIds(JSContext* cx, DeviceAccelerationInitAtoms* atomsCache);

public:
  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value");

  bool
  Init(const nsAString& aJSON);

  bool
  ToObjectInternal(JSContext* cx, JS::MutableHandle<JS::Value> rval) const;

  void
  TraceDictionary(JSTracer* trc);

  void
  operator=(const DeviceAccelerationInit& aOther);
};

namespace binding_detail {
struct FastDeviceAccelerationInit : public DeviceAccelerationInit
{
  inline FastDeviceAccelerationInit()
    : DeviceAccelerationInit(42)
  {
    // Doesn't matter what int we pass to the parent constructor
  }
};
} // namespace binding_detail


struct DeviceRotationRateInit : public DictionaryBase
{
  Nullable<double> mAlpha;
  Nullable<double> mBeta;
  Nullable<double> mGamma;

  DeviceRotationRateInit();

  explicit inline DeviceRotationRateInit(const DeviceRotationRateInit& aOther)
  {
    *this = aOther;
  }

protected:
  explicit inline DeviceRotationRateInit(int )
  {
    // Do nothing here; this is used by our "Fast" subclass
  }

private:
  static bool
  InitIds(JSContext* cx, DeviceRotationRateInitAtoms* atomsCache);

public:
  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value");

  bool
  Init(const nsAString& aJSON);

  bool
  ToObjectInternal(JSContext* cx, JS::MutableHandle<JS::Value> rval) const;

  void
  TraceDictionary(JSTracer* trc);

  void
  operator=(const DeviceRotationRateInit& aOther);
};

namespace binding_detail {
struct FastDeviceRotationRateInit : public DeviceRotationRateInit
{
  inline FastDeviceRotationRateInit()
    : DeviceRotationRateInit(42)
  {
    // Doesn't matter what int we pass to the parent constructor
  }
};
} // namespace binding_detail


struct DeviceMotionEventInit : public EventInit
{
  DeviceAccelerationInit mAcceleration;
  DeviceAccelerationInit mAccelerationIncludingGravity;
  Nullable<double> mInterval;
  DeviceRotationRateInit mRotationRate;

  DeviceMotionEventInit();

  explicit inline DeviceMotionEventInit(const DeviceMotionEventInit& aOther)
  {
    *this = aOther;
  }

protected:
  explicit inline DeviceMotionEventInit(int )
  {
    // Do nothing here; this is used by our "Fast" subclass
  }

private:
  static bool
  InitIds(JSContext* cx, DeviceMotionEventInitAtoms* atomsCache);

public:
  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value");

  bool
  Init(const nsAString& aJSON);

  bool
  ToObjectInternal(JSContext* cx, JS::MutableHandle<JS::Value> rval) const;

  void
  TraceDictionary(JSTracer* trc);

  void
  operator=(const DeviceMotionEventInit& aOther);
};

namespace binding_detail {
struct FastDeviceMotionEventInit : public DeviceMotionEventInit
{
  inline FastDeviceMotionEventInit()
    : DeviceMotionEventInit(42)
  {
    // Doesn't matter what int we pass to the parent constructor
  }
};
} // namespace binding_detail


namespace DeviceAccelerationBinding {

  typedef mozilla::dom::DeviceAcceleration NativeType;

  // We declare this as an array so that retrieving a pointer to this
  // binding's property hooks only requires compile/link-time resolvable
  // address arithmetic.  Declaring it as a pointer instead would require
  // doing a run-time load to fetch a pointer to this binding's property
  // hooks.  And then structures which embedded a pointer to this structure
  // would require a run-time load for proper initialization, which would
  // then induce static constructors.  Lots of static constructors.
  extern const NativePropertyHooks sNativePropertyHooks[];

  const JSClass*
  GetJSClass();

  JSObject*
  Wrap(JSContext* aCx, mozilla::dom::DeviceAcceleration* aObject, nsWrapperCache* aCache);

  template <class T>
  inline JSObject* Wrap(JSContext* aCx, T* aObject)
  {
    return Wrap(aCx, aObject, aObject);
  }

  void
  CreateInterfaceObjects(JSContext* aCx, JS::Handle<JSObject*> aGlobal, ProtoAndIfaceCache& aProtoAndIfaceCache, bool aDefineOnGlobal);

  JS::Handle<JSObject*>
  GetProtoObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal);

} // namespace DeviceAccelerationBinding



namespace DeviceMotionEventBinding {

  typedef mozilla::dom::DeviceMotionEvent NativeType;

  // We declare this as an array so that retrieving a pointer to this
  // binding's property hooks only requires compile/link-time resolvable
  // address arithmetic.  Declaring it as a pointer instead would require
  // doing a run-time load to fetch a pointer to this binding's property
  // hooks.  And then structures which embedded a pointer to this structure
  // would require a run-time load for proper initialization, which would
  // then induce static constructors.  Lots of static constructors.
  extern const NativePropertyHooks sNativePropertyHooks[];

  JSObject*
  DefineDOMInterface(JSContext* aCx, JS::Handle<JSObject*> aGlobal, JS::Handle<jsid> id, bool aDefineOnGlobal);

  const JSClass*
  GetJSClass();

  JSObject*
  Wrap(JSContext* aCx, mozilla::dom::DeviceMotionEvent* aObject, nsWrapperCache* aCache);

  template <class T>
  inline JSObject* Wrap(JSContext* aCx, T* aObject)
  {
    return Wrap(aCx, aObject, aObject);
  }

  void
  CreateInterfaceObjects(JSContext* aCx, JS::Handle<JSObject*> aGlobal, ProtoAndIfaceCache& aProtoAndIfaceCache, bool aDefineOnGlobal);

  JS::Handle<JSObject*>
  GetProtoObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal);

  JS::Handle<JSObject*>
  GetConstructorObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal, bool aDefineOnGlobal = true);

} // namespace DeviceMotionEventBinding



namespace DeviceRotationRateBinding {

  typedef mozilla::dom::DeviceRotationRate NativeType;

  // We declare this as an array so that retrieving a pointer to this
  // binding's property hooks only requires compile/link-time resolvable
  // address arithmetic.  Declaring it as a pointer instead would require
  // doing a run-time load to fetch a pointer to this binding's property
  // hooks.  And then structures which embedded a pointer to this structure
  // would require a run-time load for proper initialization, which would
  // then induce static constructors.  Lots of static constructors.
  extern const NativePropertyHooks sNativePropertyHooks[];

  const JSClass*
  GetJSClass();

  JSObject*
  Wrap(JSContext* aCx, mozilla::dom::DeviceRotationRate* aObject, nsWrapperCache* aCache);

  template <class T>
  inline JSObject* Wrap(JSContext* aCx, T* aObject)
  {
    return Wrap(aCx, aObject, aObject);
  }

  void
  CreateInterfaceObjects(JSContext* aCx, JS::Handle<JSObject*> aGlobal, ProtoAndIfaceCache& aProtoAndIfaceCache, bool aDefineOnGlobal);

  JS::Handle<JSObject*>
  GetProtoObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal);

} // namespace DeviceRotationRateBinding



} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_DeviceMotionEventBinding_h
