/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set ts=8 sts=2 et sw=2 tw=80: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMCSSRule.idl"

[scriptable, uuid(5f9f2068-743b-42e3-becb-10ffa994d1e3)]
interface nsIDOMCSSCounterStyleRule : nsIDOMCSSRule
{
  attribute DOMString name;
  attribute DOMString system;
  attribute DOMString symbols;
  attribute DOMString additiveSymbols;
  attribute DOMString negative;
  attribute DOMString prefix;
  attribute DOMString suffix;
  attribute DOMString range;
  attribute DOMString pad;
  attribute DOMString speakAs;
  attribute DOMString fallback;
};
