/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"
#include "mozISpellI18NUtil.idl"

[scriptable, uuid(AEB8936F-219C-4D3C-8385-D9382DAA551A)]

/**
 * This interface contains various I18N related code used in various places
 * by the spell checker.
 */

interface mozISpellI18NManager : nsISupports {
  /**
   * Get a mozISpellI18NUtil interface that best matches the given language.
   */
  mozISpellI18NUtil getUtil(in wstring language);
};
